/* global Bdf,Ficheform,$$,Fql */

Ficheform.ThesaurusInclude = {};

Ficheform.ThesaurusInclude.init = function () {
    Bdf.Api.init();
    Bdf.$({ficheformEntry: "thesaurus-include"}).each(function (index, element) {
        let $entry = $(element);
        let entryKey = element.dataset.ficheformKey;
        let subsetName = Bdf.getSubsetName($entry);
        let inputId = Bdf.$($entry, {ficheformRole: "textinput"})[0].id;
        let buttonId = Bdf.generateId();
        let isExternalSource, action, locKey;
        if ($entry.data("ficheformExternalsource")) {
            isExternalSource = true;
            action = "action-ficheform-ExternalSource";
            locKey = "_ link.pioche.externalsource";
        } else {
            isExternalSource = false;
            action = "action-ficheform-Motcle";
            locKey = "_ link.pioche.motcle_one";
        }
        Bdf.$($entry, {ficheformRole: "motcle-input"}).append(Bdf.render("ficheform:pioche", {
            buttonId: buttonId,
            action: action,
            locKey: locKey,
            locValue:  Bdf.Loc.get("subset:thesaurus_" + subsetName),
            noTab: Ficheform.getOption("noTab")
        }));
        $$(buttonId).click(function () {
            _pioche(entryKey, subsetName, inputId, isExternalSource);
        });
        if (Bdf.$($entry, {ficheformRole: "cell-enumeration"}).length > 0) {
            _initFicheStyle(element, subsetName, isExternalSource);
        }
    });

    
    function _pioche(entryKey, subsets, inputId, isExternalSource) {
        let page;
        if (isExternalSource) {
            page = "externalsource";
        } else {
            page = "motcle";
        }
        let $entry = Ficheform.$entry(entryKey);
        let limit = $entry.data("ficheformLimit");
        if (!limit) {
            limit = -1;
        }
        let url = "pioche?page=" + page + "&subsets=" + subsets + "&appelant=" + inputId + "&limit=" + limit;
        url = url + "&croisement=corpus_" + Ficheform.getCurrentCorpus($entry);
        let wanted = $entry.data("ficheformWanted");
        if (wanted) {
            url = url + "&wanted=" +  wanted;
        }
        Ficheform.openDialog(url,"pioche_"+ entryKey,430,510);
    }
    
    function _initFicheStyle(element, thesaurusName, isExternalSource) {
        let hereMap = new Map();
        let entryKey = element.dataset.ficheformKey;
        let idalphaStyle = element.dataset.ficheformIdalphastyle;
        let withPoids = element.dataset.ficheformWithpoids;
        let $entry = $(element);
        __initSearchArea();
        if (withPoids) {
            Ficheform.initPoidsInput(Bdf.$($entry, {ficheformRole: "cell-enumeration"}));
        }
        let $choices = Bdf.$($entry, {ficheformRole: "item-choices"});
        Bdf.$($choices, {ficheformRole: "item-choice"}).each(function (index, element) {
            __initChoice(element);
        });
        
        function __initChoice(element) {
            let motcleId = element.dataset.ficheformItemId;
            hereMap.set(parseInt(motcleId), true);
            let genId = Bdf.generateId();
            Bdf.$(element, {_element: "label"}).after(Bdf.render("ficheform:include/choicebuttons", {
                genId: genId,
                noTab: Ficheform.getOption("noTab")
            }));
            Ficheform.initOrderButtons(genId, element);
        }
        
        function __showEnumeration() {
            Bdf.$($entry, {ficheformRole: "cell-enumeration"}).removeClass("hidden");
        }
        
        function __initSearchArea() {
            let searchInputId = Bdf.generateId();
            let searchAnchorId = Bdf.generateId();
            Bdf.$($entry, {ficheformRole: "searchcontainer"}).prepend(Bdf.render("ficheform:include/searcharea", {
                noTab: Ficheform.getOption("noTab"),
                createButtonId: "",
                searchInputId: searchInputId,
                withCreate: false,
                createTitle: ""
            })).after(Bdf.render("ficheform:include/searchanchor", {
                searchAnchorId: searchAnchorId
            }));
            Ficheform.initSearchInput(searchInputId, searchAnchorId, function (input) {
                Ficheform.delayFunction(input.id, function () {
                    let query = input.value.trim();
                    if (query) {
                        if (isExternalSource) {
                           ___searchExternalSource(query);
                        } else {
                           ___searchThesaurus(query);
                        }
                    } else {
                        $$(searchAnchorId).html("");
                    }
                });
            });
            Bdf.onClick($$(searchAnchorId), {action: "itemadd"}, function () {
                ___addChoice(thesaurusName, this.dataset.id);
                $$(searchAnchorId).html("");
                $$(searchInputId).val("");
                return false;
            });
            
            function ___addChoice(thesaurusName, id) {
                let requestParameters = {
                    properties: "special:phrases"
                };
                if (isExternalSource) {
                    requestParameters.cmd = "MotcleCheck";
                }
                Bdf.api.loadMotcle({
                    thesaurus: thesaurusName,
                    id: id,
                    requestParameters: requestParameters,
                    callback: function (data) {
                        let motcle = data.motcle;
                        __showEnumeration();
                        let checkboxValue = id;
                        if (motcle.idalpha) {
                            checkboxValue = motcle.idalpha;
                        }
                        let title = Ficheform.concatMotcleTitle(motcle, idalphaStyle);
                        $choices.append(Bdf.render("ficheform:include/itemchoice", {
                            itemId: id,
                            checkboxName: entryKey,
                            checkboxValue: checkboxValue,
                            checkboxId: Bdf.generateId(),
                            withPoids: withPoids,
                            number: motcle.properties.number,
                            title: title
                        }));
                        __initChoice(Bdf.$($choices, {ficheformItemId: id})[0]);
                        Ficheform.signalChange();
                    }
                });
            }
            
            function ___searchExternalSource(query) {
                $.ajax({
                    url: Bdf.api.getActionUrl("pioche"),
                    method: "GET",
                    dataType: "json",
                    data: {
                        thesaurus: thesaurusName,
                        json: "externalitem",
                        q: query,
                        limit: 50,
                        sort: "id-desc"
                    },
                    success: function (data, textStatus) {
                        let externalArray = data.array;
                        if (externalArray.length === 0) {
                            $$(searchAnchorId).html(Bdf.render("ficheform:noresult"));
                        } else {
                            let first = true;
                            for(let externalItem of externalArray) {
                                if (hereMap.has(externalItem.id)) {
                                    externalItem._here = true;
                                } else if (first) {
                                    externalItem._selected = first;
                                    first = false;
                                }
                                externalItem._title = externalItem.id + " – " + externalItem.title;
                            }
                            $$(searchAnchorId).html(Bdf.render("ficheform:include/searchlist", {array: externalArray}));
                        }
                    }
                });
            }
            
            function ___searchThesaurus(query) {
                Bdf.api.loadMotcleArray({
                    motcleQuery: {
                        thesaurus: thesaurusName,
                        content: {
                            scope: Fql.SCOPE_IDALPHA_WITH,
                            q: query,
                            type: Fql.QTYPE_SIMPLE
                        },
                        status: "active"
                    },
                    requestParameters: {
                        limit: 50
                    },
                    callback: function (data) {
                        let motcleArray = data.array;
                        if (motcleArray.length === 0) {
                            $$(searchAnchorId).html(Bdf.render("ficheform:noresult"));
                        } else {
                            let first = true;
                            for(let motcle of motcleArray) {
                                if (hereMap.has(motcle.id)) {
                                    motcle._here = true;
                                } else if (first) {
                                    motcle._selected = first;
                                    first = false;
                                }
                                motcle._title = Ficheform.concatMotcleTitle(motcle, idalphaStyle);
                            }
                            $$(searchAnchorId).html(Bdf.render("ficheform:include/searchlist", {array: motcleArray}));
                        }
                    }
                });
            }

        }
    }
    
};

